/*
 * Copyright 2013 Michael Osipov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.michaelo.tomcat.extras.listeners;

import java.io.Closeable;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.naming.Context;
import javax.naming.NamingException;

import net.sf.michaelo.tomcat.extras.utils.ContextUtils;

import org.apache.catalina.Lifecycle;
import org.apache.catalina.LifecycleEvent;
import org.apache.catalina.LifecycleListener;
import org.apache.catalina.deploy.ContextResource;
import org.apache.commons.lang3.reflect.MethodUtils;
import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;
import org.apache.naming.ContextBindings;

/**
 * A listener which closes all JNDI resources which do have a zero arguments
 * close method when the context is shut down. Otherwise resources will remain
 * stale until GC.
 * <p>
 * If a resource implements {@link Closeable} it will be used otherwise
 * reflection will be used to check and invoke the close method.
 * </p>
 * <p>
 * It addresses bug <a
 * href="https://issues.apache.org/bugzilla/show_bug.cgi?id=25060">25060</a>
 * which was fixed in Tomcat 7 only. See <a
 * href="http://www.mail-archive.com/users@tomcat.apache.org/msg88156.html"
 * >mailing list</a> and <a href=
 * "http://stackoverflow.com/questions/5407525/should-i-close-jndi-obtained-data-source"
 * >Stack Overflow</a> for further discussion.
 * </p>
 *
 * @version $Id: CloseableResourceListener.java 21 2013-03-20 20:48:43Z michael-o $
 */
public class CloseableResourceListener implements LifecycleListener {

	private static final Log logger = LogFactory
			.getLog(CloseableResourceListener.class);

	private static final String CLOSE_METHOD_NAME = "close";

	@Override
	public void lifecycleEvent(LifecycleEvent le) {

		if (!(le.getLifecycle() instanceof org.apache.catalina.Context))
			return;

		if (le.getType().equals(Lifecycle.STOP_EVENT)) {

			org.apache.catalina.Context catalinaContext = (org.apache.catalina.Context) le
					.getLifecycle();
			ContextResource[] resources = catalinaContext.getNamingResources()
					.findResources();

			String contextName = ContextUtils.getName(catalinaContext);

			Context context;
			try {
				context = ContextBindings.getClassLoader();
				context = (Context) context.lookup("comp/env");
			} catch (NamingException e) {
				logger.warn(
						String.format(
								"Failed to retrieve JNDI naming context for context '%s' so no cleanup can be performed",
								contextName), e);
				return;
			}

			for (ContextResource contextResource : resources) {
				String name = contextResource.getName();
				Object resource;

				try {
					resource = context.lookup(name);
				} catch (NamingException e) {
					logger.warn(String
							.format("Failed to retrieve JNDI resource '%s' in context '%s' so no cleanup can be performed",
									name, contextName), e);
					continue;
				}

				if (resource instanceof Closeable) {
					Closeable closeable = (Closeable) resource;
					try {
						closeable.close();
					} catch (IOException e) {
						logger.warn(
								String.format(
										"Failed to invoke close method on JNDI resource '%s' in context '%s' so no cleanup can be performed",
										name, contextName), e);
					}
				} else {
					Method closeMethod = MethodUtils.getAccessibleMethod(
							resource.getClass(), CLOSE_METHOD_NAME);
					if (closeMethod != null) {
						try {
							closeMethod.invoke(resource);
						} catch (InvocationTargetException e) {
							logger.warn(
									String.format(
											"Failed to invoke close method on JNDI resource '%s' in context '%s' so no cleanup can be performed",
											name, contextName), e.getTargetException());
						} catch (Exception e) {
							logger.warn(
									String.format(
											"Failed to invoke close method on JNDI resource '%s' in context '%s' so no cleanup can be performed",
											name, contextName), e);
						}
					} else
						logger.debug(String
								.format("JNDI resource '%s' in context '%s' has no close method so no cleanup can be performed",
										name, contextName));
				}

			}

		}

	}

}
