/*
 * Copyright 2013 Michael Osipov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.michaelo.tomcat.extras.listeners;

import net.sf.michaelo.tomcat.extras.utils.ContextUtils;

import org.apache.catalina.Context;
import org.apache.catalina.Lifecycle;
import org.apache.catalina.LifecycleEvent;
import org.apache.catalina.LifecycleListener;
import org.apache.catalina.deploy.ContextEnvironment;
import org.apache.commons.lang3.StringUtils;
import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;

/**
 * A listener which creates an environment entry with the name of the current
 * context to populate Logback's context name fully dynamic in a webapp's
 * {@code logback.xml} file.
 * <p>
 * The default entry name is {@code logback/contextName}.
 * </p>
 *
 * @version $Id: LogbackContextNameListener.java 23 2013-03-20 21:13:25Z michael-o $
 */
public class LogbackContextNameListener implements LifecycleListener {

	private static final Log logger = LogFactory
			.getLog(LogbackContextNameListener.class);
	private Context context;

	private String name = "logback/contextName";

	@Override
	public void lifecycleEvent(LifecycleEvent le) {

		if (le.getLifecycle() instanceof Context)
			context = (Context) le.getLifecycle();
		else
			return;

		String contextName = ContextUtils.getName(context);
		if (le.getType().equals(Lifecycle.START_EVENT)) {
			// Check whether env entry is already set and remove it as advised
			// by Konstantin Kolinko
			ContextEnvironment ce = context.getNamingResources()
					.findEnvironment(name);
			if (ce != null)
				context.getNamingResources().removeEnvironment(name);

			ce = new ContextEnvironment();
			ce.setName(name);
			ce.setOverride(false);
			ce.setType("java.lang.String");
			ce.setValue(contextName);
			if (logger.isDebugEnabled())
				logger.debug(String
						.format("Adding Logback context name env entry '%s' to context '%s'",
								name, contextName));
			context.getNamingResources().addEnvironment(ce);
		}

	}

	public void setName(String name) {
		if (StringUtils.isEmpty(name))
			throw new IllegalArgumentException(
					"Parameter 'name' cannot be empty");

		this.name = name;
	}

}
